@@ -133,7 +133,7 @@ fn parse(text: &str) -> Parse {
             self.skip_ws();
             self.expect(IDENTIFIER);
             self.skip_ws();
-            if self.tokens.pop().map(|(k, t)| (k, t)) == Some((OPERATOR, ":".to_string())) {
+            if self.tokens.pop() == Some((OPERATOR, ":".to_string())) {
                 self.builder.token(OPERATOR.into(), ":");
             } else {
                 self.error("expected ':'".into());
@@ -161,6 +161,10 @@ fn parse(text: &str) -> Parse {
         fn parse_assignment(&mut self) {
             self.builder.start_node(VARIABLE.into());
             self.skip_ws();
+            if self.tokens.last() == Some(&(IDENTIFIER, "export".to_string())) {
+                self.expect(IDENTIFIER);
+                self.skip_ws();
+            }
             self.expect(IDENTIFIER);
             self.skip_ws();
             self.expect(OPERATOR);
@@ -177,7 +181,13 @@ fn parse(text: &str) -> Parse {
                     Some((OPERATOR, ":")) => {
                         self.parse_rule();
                     }
-                    Some((OPERATOR, "?=")) | Some((OPERATOR, "=")) | Some((OPERATOR, ":=")) | Some((OPERATOR, "::=")) | Some((OPERATOR, ":::=")) | Some((OPERATOR, "+=")) | Some((OPERATOR, "!=")) => {
+                    Some((OPERATOR, "?="))
+                    | Some((OPERATOR, "="))
+                    | Some((OPERATOR, ":="))
+                    | Some((OPERATOR, "::="))
+                    | Some((OPERATOR, ":::="))
+                    | Some((OPERATOR, "+="))
+                    | Some((OPERATOR, "!=")) => {
                         self.parse_assignment();
                     }
                     Some((NEWLINE, _)) => {
@@ -214,8 +224,15 @@ fn parse(text: &str) -> Parse {
             self.tokens.last().map(|(kind, _)| *kind)
         }
 
-        fn find(&self, finder: impl FnMut(&&(SyntaxKind, String)) -> bool) -> Option<(SyntaxKind, &str)> {
-            self.tokens.iter().rev().find(finder).map(|(kind, text)| (*kind, text.as_str()))
+        fn find(
+            &self,
+            finder: impl FnMut(&&(SyntaxKind, String)) -> bool,
+        ) -> Option<(SyntaxKind, &str)> {
+            self.tokens
+                .iter()
+                .rev()
+                .find(finder)
+                .map(|(kind, text)| (*kind, text.as_str()))
         }
 
         fn expect(&mut self, expected: SyntaxKind) {
@@ -290,9 +307,9 @@ macro_rules! ast_node {
             }
         }
 
-        impl ToString for $ast {
-            fn to_string(&self) -> String {
-                self.0.text().to_string()
+        impl core::fmt::Display for $ast {
+            fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> Result<(), core::fmt::Error> {
+                write!(f, "{}", self.0.text())
             }
         }
     };
@@ -305,17 +322,22 @@ ast_node!(VariableDefinition, VARIABLE);
 
 impl VariableDefinition {
     pub fn name(&self) -> Option<String> {
-        self.syntax().children_with_tokens().find_map(|it| it.as_token().and_then(|it| {
-            if it.kind() == IDENTIFIER {
-                Some(it.text().to_string())
-            } else {
-                None
-            }
-        }))
+        self.syntax().children_with_tokens().find_map(|it| {
+            it.as_token().and_then(|it| {
+                if it.kind() == IDENTIFIER && it.text() != "export" {
+                    Some(it.text().to_string())
+                } else {
+                    None
+                }
+            })
+        })
     }
 
     pub fn raw_value(&self) -> Option<String> {
-        self.syntax().children().find(|it| it.kind() == EXPR).map(|it| it.text().to_string())
+        self.syntax()
+            .children()
+            .find(|it| it.kind() == EXPR)
+            .map(|it| it.text().to_string())
     }
 }
 
@@ -346,9 +368,12 @@ impl Makefile {
     }
 
     pub fn rules(&self) -> impl Iterator<Item = Rule> {
-        self.syntax()
-            .children()
-            .filter_map(Rule::cast)
+        self.syntax().children().filter_map(Rule::cast)
+    }
+
+    pub fn rules_by_target<'a>(&'a self, target: &'a str) -> impl Iterator<Item = Rule> + 'a {
+        self.rules()
+            .filter(move |rule| rule.targets().any(|t| t == target))
     }
 
     pub fn variable_definitions(&self) -> impl Iterator<Item = VariableDefinition> {
@@ -366,8 +391,9 @@ impl Makefile {
         builder.finish_node();
 
         let syntax = SyntaxNode::new_root(builder.finish()).clone_for_update();
-        let pos = self.0.children().count();
-        self.0.splice_children(pos..pos, vec![syntax.clone().into()]);
+        let pos = self.0.children_with_tokens().count();
+        self.0
+            .splice_children(pos..pos, vec![syntax.clone().into()]);
         Rule(syntax)
     }
 }
@@ -384,7 +410,81 @@ impl Rule {
         self.syntax()
             .children()
             .find(|it| it.kind() == EXPR)
-            .into_iter().flat_map(|it| it.children_with_tokens().filter_map(|it| it.as_token().and_then(|t| if t.kind() == IDENTIFIER { Some(t.text().to_string()) } else { None })))
+            .into_iter()
+            .flat_map(|it| {
+                it.children_with_tokens().filter_map(|it| {
+                    it.as_token().and_then(|t| {
+                        if t.kind() == IDENTIFIER {
+                            Some(t.text().to_string())
+                        } else {
+                            None
+                        }
+                    })
+                })
+            })
+    }
+
+    pub fn recipes(&self) -> impl Iterator<Item = String> {
+        self.syntax()
+            .children()
+            .filter(|it| it.kind() == RECIPE)
+            .flat_map(|it| {
+                it.children_with_tokens().filter_map(|it| {
+                    it.as_token().and_then(|t| {
+                        if t.kind() == TEXT {
+                            Some(t.text().to_string())
+                        } else {
+                            None
+                        }
+                    })
+                })
+            })
+    }
+
+    pub fn replace_command(&self, i: usize, line: &str) {
+        // Find the RECIPE with index i, then replace the line in it
+        let index = self
+            .syntax()
+            .children()
+            .filter(|it| it.kind() == RECIPE)
+            .nth(i)
+            .expect("index out of bounds")
+            .index();
+
+        let mut builder = GreenNodeBuilder::new();
+        builder.start_node(RECIPE.into());
+        builder.token(INDENT.into(), "\t");
+        builder.token(TEXT.into(), line);
+        builder.token(NEWLINE.into(), "\n");
+        builder.finish_node();
+
+        let syntax = SyntaxNode::new_root(builder.finish()).clone_for_update();
+        self.0
+            .splice_children(index..index + 1, vec![syntax.into()]);
+    }
+
+    pub fn push_command(&self, line: &str) {
+        // Find the latest RECIPE entry, then append the new line after it.
+        let index = self
+            .0
+            .children_with_tokens()
+            .filter(|it| it.kind() == RECIPE)
+            .last();
+
+        let index = index.map_or_else(
+            || self.0.children_with_tokens().count(),
+            |it| it.index() + 1,
+        );
+
+        let mut builder = GreenNodeBuilder::new();
+        builder.start_node(RECIPE.into());
+        builder.token(INDENT.into(), "\t");
+        builder.token(TEXT.into(), line);
+        builder.token(NEWLINE.into(), "\n");
+        builder.finish_node();
+        let syntax = SyntaxNode::new_root(builder.finish()).clone_for_update();
+
+        self.0.splice_children(index..index, vec![syntax.into()]);
     }
 }
 
@@ -407,19 +507,23 @@ impl FromStr for Makefile {
     }
 }
 
-#[test]
-fn test_parse_simple() {
-    const SIMPLE: &str = r#"VARIABLE = value
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_parse_simple() {
+        const SIMPLE: &str = r#"VARIABLE = value
 
 rule: dependency
 	command
 "#;
-    let parsed = parse(SIMPLE);
-    assert_eq!(parsed.errors, Vec::<String>::new());
-    let node = parsed.syntax();
-    assert_eq!(
-        format!("{:#?}", node),
-        r#"ROOT@0..44
+        let parsed = parse(SIMPLE);
+        assert_eq!(parsed.errors, Vec::<String>::new());
+        let node = parsed.syntax();
+        assert_eq!(
+            format!("{:#?}", node),
+            r#"ROOT@0..44
   VARIABLE@0..17
     IDENTIFIER@0..8 "VARIABLE"
     WHITESPACE@8..9 " "
@@ -441,35 +545,68 @@ rule: dependency
       TEXT@36..43 "command"
       NEWLINE@43..44 "\n"
 "#
-    );
+        );
 
-    let root = parsed.root().clone_for_update();
+        let root = parsed.root().clone_for_update();
 
-    let mut rules = root.rules().collect::<Vec<_>>();
-    assert_eq!(rules.len(), 1);
-    let rule = rules.pop().unwrap();
-    assert_eq!(rule.targets().collect::<Vec<_>>(), vec!["rule"]);
-    assert_eq!(rule.prerequisites().collect::<Vec<_>>(), vec!["dependency"]);
+        let mut rules = root.rules().collect::<Vec<_>>();
+        assert_eq!(rules.len(), 1);
+        let rule = rules.pop().unwrap();
+        assert_eq!(rule.targets().collect::<Vec<_>>(), vec!["rule"]);
+        assert_eq!(rule.prerequisites().collect::<Vec<_>>(), vec!["dependency"]);
+        assert_eq!(rule.recipes().collect::<Vec<_>>(), vec!["command"]);
 
-    let mut variables = root.variable_definitions().collect::<Vec<_>>();
-    assert_eq!(variables.len(), 1);
-    let variable = variables.pop().unwrap();
-    assert_eq!(variable.name(), Some("VARIABLE".to_string()));
-    assert_eq!(variable.raw_value(), Some("value".to_string()));
-}
+        let mut variables = root.variable_definitions().collect::<Vec<_>>();
+        assert_eq!(variables.len(), 1);
+        let variable = variables.pop().unwrap();
+        assert_eq!(variable.name(), Some("VARIABLE".to_string()));
+        assert_eq!(variable.raw_value(), Some("value".to_string()));
+    }
 
-#[test]
-fn test_parse_multiple_prerequisites() {
-    const MULTIPLE_PREREQUISITES: &str = r#"rule: dependency1 dependency2
+    #[test]
+    fn test_parse_export_assign() {
+        const EXPORT: &str = r#"export VARIABLE := value
+"#;
+        let parsed = parse(EXPORT);
+        assert_eq!(parsed.errors, Vec::<String>::new());
+        let node = parsed.syntax();
+        assert_eq!(
+            format!("{:#?}", node),
+            r#"ROOT@0..25
+  VARIABLE@0..25
+    IDENTIFIER@0..6 "export"
+    WHITESPACE@6..7 " "
+    IDENTIFIER@7..15 "VARIABLE"
+    WHITESPACE@15..16 " "
+    OPERATOR@16..18 ":="
+    WHITESPACE@18..19 " "
+    EXPR@19..24
+      IDENTIFIER@19..24 "value"
+    NEWLINE@24..25 "\n"
+"#
+        );
+
+        let root = parsed.root().clone_for_update();
+
+        let mut variables = root.variable_definitions().collect::<Vec<_>>();
+        assert_eq!(variables.len(), 1);
+        let variable = variables.pop().unwrap();
+        assert_eq!(variable.name(), Some("VARIABLE".to_string()));
+        assert_eq!(variable.raw_value(), Some("value".to_string()));
+    }
+
+    #[test]
+    fn test_parse_multiple_prerequisites() {
+        const MULTIPLE_PREREQUISITES: &str = r#"rule: dependency1 dependency2
 	command
 
 "#;
-    let parsed = parse(MULTIPLE_PREREQUISITES);
-    assert_eq!(parsed.errors, Vec::<String>::new());
-    let node = parsed.syntax();
-    assert_eq!(
-        format!("{:#?}", node),
-        r#"ROOT@0..40
+        let parsed = parse(MULTIPLE_PREREQUISITES);
+        assert_eq!(parsed.errors, Vec::<String>::new());
+        let node = parsed.syntax();
+        assert_eq!(
+            format!("{:#?}", node),
+            r#"ROOT@0..40
   RULE@0..40
     IDENTIFIER@0..4 "rule"
     OPERATOR@4..5 ":"
@@ -484,23 +621,67 @@ fn test_parse_multiple_prerequisites() {
       TEXT@31..38 "command"
       NEWLINE@38..39 "\n"
     NEWLINE@39..40 "\n"
-"#);
-    let root = parsed.root().clone_for_update();
+"#
+        );
+        let root = parsed.root().clone_for_update();
 
-    let rule = root.rules().next().unwrap();
-    assert_eq!(rule.targets().collect::<Vec<_>>(), vec!["rule"]);
-    assert_eq!(
-        rule.prerequisites().collect::<Vec<_>>(),
-        vec!["dependency1", "dependency2"]
-    );
-}
-
-#[test]
-fn test_add_rule() {
-    let mut makefile = Makefile::new();
-    let rule = makefile.add_rule("rule");
-    assert_eq!(rule.targets().collect::<Vec<_>>(), vec!["rule"]);
-    assert_eq!(rule.prerequisites().collect::<Vec<_>>(), Vec::<String>::new());
-
-    assert_eq!(makefile.to_string(), "rule:\n");
+        let rule = root.rules().next().unwrap();
+        assert_eq!(rule.targets().collect::<Vec<_>>(), vec!["rule"]);
+        assert_eq!(
+            rule.prerequisites().collect::<Vec<_>>(),
+            vec!["dependency1", "dependency2"]
+        );
+        assert_eq!(rule.recipes().collect::<Vec<_>>(), vec!["command"]);
+    }
+
+    #[test]
+    fn test_add_rule() {
+        let mut makefile = Makefile::new();
+        let rule = makefile.add_rule("rule");
+        assert_eq!(rule.targets().collect::<Vec<_>>(), vec!["rule"]);
+        assert_eq!(
+            rule.prerequisites().collect::<Vec<_>>(),
+            Vec::<String>::new()
+        );
+
+        assert_eq!(makefile.to_string(), "rule:\n");
+    }
+
+    #[test]
+    fn test_push_command() {
+        let mut makefile = Makefile::new();
+        let rule = makefile.add_rule("rule");
+        rule.push_command("command");
+        assert_eq!(rule.recipes().collect::<Vec<_>>(), vec!["command"]);
+
+        assert_eq!(makefile.to_string(), "rule:\n\tcommand\n");
+
+        rule.push_command("command2");
+        assert_eq!(
+            rule.recipes().collect::<Vec<_>>(),
+            vec!["command", "command2"]
+        );
+
+        assert_eq!(makefile.to_string(), "rule:\n\tcommand\n\tcommand2\n");
+    }
+
+    #[test]
+    fn test_replace_command() {
+        let mut makefile = Makefile::new();
+        let rule = makefile.add_rule("rule");
+        rule.push_command("command");
+        rule.push_command("command2");
+        assert_eq!(
+            rule.recipes().collect::<Vec<_>>(),
+            vec!["command", "command2"]
+        );
+
+        rule.replace_command(0, "new command");
+        assert_eq!(
+            rule.recipes().collect::<Vec<_>>(),
+            vec!["new command", "command2"]
+        );
+
+        assert_eq!(makefile.to_string(), "rule:\n\tnew command\n\tcommand2\n");
+    }
 }
